@@ -1,6 +1,6 @@
 # Dejanews.pm
 # Copyright (C) 1998 by Martin Thurn
-# $Id: Dejanews.pm,v 1.11 1999/10/05 19:42:21 mthurn Exp $
+# $Id: Dejanews.pm,v 1.16 2000/01/18 15:19:43 mthurn Exp $
 
 =head1 NAME
 
@@ -50,6 +50,20 @@ been tested):
                           'fromdate' => 'Jan 1 1997',
                           'todate'   => 'Dec 31 1997', } );
 
+=head1 NOTES
+
+In the SearchResults, the description field contains the forum name
+and author's name (as reported by www.deja.com) in the following
+format: "Newsgroup: comp.lang.perl; Author: Martin Thurn"
+
+=head1 CAVEATS
+
+Names of newsgroups ("forums") are unceremoniously truncated to 21
+characters by www.deja.com.
+
+Titles ("Subject:" line of messages) are unceremoniously truncated to 28
+characters by www.deja.com.
+
 =head1 SEE ALSO
 
 To make new back-ends, see L<WWW::Search>.
@@ -77,6 +91,18 @@ MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
 =head1 VERSION HISTORY
 
+=head2 2.07, 2000-01-18
+
+handle www.deja.com's new output format
+
+=head2 2.06, 1999-12-07
+
+new test cases, pod update, ignore deja links, etc.
+
+=head2 2.04, 1999-12-06
+
+handle www.deja.com's new output format
+
 =head2 2.03, 1999-10-05
 
 now uses hash_to_cgi_string()
@@ -119,7 +145,7 @@ require Exporter;
 @EXPORT = qw();
 @EXPORT_OK = qw();
 @ISA = qw(WWW::Search Exporter);
-$VERSION = '2.03';
+$VERSION = '2.07';
 
 use Carp ();
 use WWW::Search(generic_option);
@@ -128,8 +154,9 @@ require WWW::SearchResult;
 $MAINTAINER = 'Martin Thurn <MartinThurn@iname.com>';
 $TEST_CASES = <<"ENDTESTCASES";
 &test('Dejanews', '$MAINTAINER', 'zero', \$bogus_query, \$TEST_EXACTLY);
-&test('Dejanews', '$MAINTAINER', 'one', 'st'.'untboy', \$TEST_RANGE, 2,99);
-&test('Dejanews', '$MAINTAINER', 'two', 'L'.'ili AND Le'.'dy', \$TEST_GREATER_THAN, 101);
+&test('Dejanews', '$MAINTAINER', 'one', 'irov'.'er', \$TEST_RANGE, 2,99);
+&test('Dejanews', '$MAINTAINER', 'two', 'L'.'ili AND Le'.'dy', \$TEST_RANGE, 101,199);
+&test('Dejanews', '$MAINTAINER', 'three', 'Jabb'.'a', \$TEST_GREATER_THAN, 201);
 ENDTESTCASES
 
 # private
@@ -151,12 +178,13 @@ sub native_setup_search
     {
     # These are the defaults:
     $self->{_options} = {
-                         'search_url' => 'http://www.deja.com/dnquery.xp',
+                         'search_url' => 'http://www.deja.com/qs.xp',
+                         'LNG' => 'ALL',
+                         'OP' => 'dnquery.xp',
                          'QRY' => $native_query,
                          'ST' => 'PS',
                          'defaultOp' => 'OR',
                          'maxhits' => $self->{'_hits_per_page'},
-                         'format' => 'delta',
                          'showsort' => 'score',
                         };
     } # if
@@ -203,35 +231,41 @@ sub native_retrieve_some
   print STDERR " *   got response\n" if $self->{'_debug'};
   $self->{'_next_url'} = undef;
   # Parse the output
-  my ($START, $HEADER, $HITS, $URL,$DATE,$FORUM, $TRAILER, $ALLDONE) = qw(ST HE HI UR DA FO TR AD);
+  my ($START, $HEADER, $HITS, $URL,$TITLE,$DATE,$FORUM,$AUTHOR, $TRAILER, $ALLDONE) = qw( ST HE HI UR TI DA FO AU TR AD );
   my $hits_found = 0;
   my $state = $START;
-  my ($hit, $sDescription);
+  my $hit;
+  my $sDate = '';
+  # The fields of each record appear in the following order: DATE, URL, TITLE, FORUM, AUTHOR
+ LINE_OF_INPUT:
   foreach ($self->split_lines($response->content())) 
     {
-    next if m/^\s*$/; # short circuit for blank lines
+    next LINE_OF_INPUT if m/^\s*$/; # short circuit for blank lines
     print STDERR " *   $state ===$_===" if 2 <= $self->{'_debug'};
     if ($state eq $START && 
-        m=messages\s[-0-9]+\sof\s(about|exactly)\s(\d+)\smatches=)
+        m=[-0-9]+\sof\s(?:about|exactly)\s(\d+)\smatches=)
       {
-      # Actual line of input is:
+      # Actual lines of input:
       #         <font face=arial,helvetica size=-1>messages 1-100 of about 2500000 matches</font>
+      # <b class="small">1-100 of exactly 3545 matches</b>
       print STDERR "count line \n" if 2 <= $self->{'_debug'};
-      $self->approximate_result_count($2);
+      $self->approximate_result_count($1);
       $state = $HITS;
       } # we're in START mode, and line has number of results
 
     elsif ($state eq $HITS &&
-           m@<a\shref=\"([^\"]+)\">Next\smatches@)
+           m@<a\shref=\"([^\"]+)\">next\s(matches|messages)@i)
       {
       # Actual line of input is:
       # <b><font face="arial,helvetica" size=2><a href="http://x1.deja.com/dnquery.xp?search=next&DBS=1&LNG=ALL&IS=Martin%20Thurn&ST=PS&offsets=db98p4x%02100&svcclass=dnserver&CONTEXT=903630253.1503199236">Next matches</a></font>
+      # <span class="small"><a href="http://x33.deja.com/dnquery.xp?search=next&LNG=ALL&IS=boba%20fett&ST=QS&offsets=db99p9%02100&CONTEXT=944504254.1945698352">next messages</a></span>
       print STDERR " found next button\n" if 2 <= $self->{'_debug'};
       # There is a "next" button on this page, therefore there are
       # indeed more results for us to go after next time.
       $self->{_next_url} = $1;
       $self->{'_next_to_retrieve'} += $self->{'_hits_per_page'};
       $state = $ALLDONE;
+      last LINE_OF_INPUT;
       }
 
 #      elsif ($state eq $HITS &&
@@ -240,7 +274,7 @@ sub native_retrieve_some
 #        print STDERR "hit score line\n" if 2 <= $self->{'_debug'};
 #        # Actual line of input:
 #        #         <b><font face="arial,helvetica" color="#ff6600">++++</font><font face="arial,helvetica" size=+1 color="#ffcc99">-</font></b><br>
-#        if (defined($hit))
+#        if (ref($hit))
 #          {
 #          push(@{$self->{cache}}, $hit);
 #          }
@@ -251,52 +285,80 @@ sub native_retrieve_some
 #        } #
 
     elsif ((($state eq $URL) || ($state eq $HITS)) && 
-           m|<a\shref=\"?([^\">]+)\"?>([^<]+)|i)
+           m|<a\shref=\"?([^\">]+)\"?>([^<]+)?|i)
       {
-      next if m/Previous\smatches/i;
+      next LINE_OF_INPUT if m/previous\s(matches|messages)/i;
       if (m/\076Power\ssearch\074/i)
         {
         $state = $ALLDONE;
-        next;
+        last LINE_OF_INPUT;
+        } # if
+      next LINE_OF_INPUT if (m/(Save|Track)\sthis\ssearch/i);
+      next LINE_OF_INPUT if (m/sort_down_x\.gif/);
+      if (m/linkback\.xp/)
+        {
+        $state = $ALLDONE;
+        last LINE_OF_INPUT;
         } # if
       print STDERR "hit url line\n" if 2 <= $self->{'_debug'};
       # Actual lines of input:
       # <td align=left><a href=http://x10.deja.com/getdoc.xp?AN=365996516&CONTEXT=899408575.427622419&hitnum=8><b>Stuffed Chewbacca</b></a><br>
       # <font face="geneva,arial" size=2 color="#999999">Previous matches</font></b>
       # For a more detailed search go to <a href="http://www.deja.com/home_ps.shtml?QRY=perlcom">Power Search</a></font></td>
+      #     <a href="http://x44.deja.com/getdoc.xp?AN=546934725&CONTEXT=944500768.1493827592&hitnum=92">
       my $sURL = $1 . '&fmt=raw';
-      my $sTitle = $2;
-      if (defined($hit))
+      if (ref($hit))
         {
+        $hit->description($sDescription);
         push(@{$self->{cache}}, $hit);
+        $sDescription = '';
         }
       $hit = new WWW::SearchResult;
       $hit->add_url($sURL);
+      $hit->change_date($sDate) if $sDate ne '';
+      $sDate = '';
       $self->{'_num_hits'}++;
       $hits_found++;
-      $hit->title($sTitle);
-      $sDescription = '';
-      $state = $FORUM;
+      $state = $TITLE;
+      }
+    elsif ($state eq $HITS && m!<td>(\d+/\d+/\d+)</td>!)
+      {
+      # Actual line of input is:
+      #     <td>12/05/1999</td>
+      print STDERR " date\n" if 2 <= $self->{'_debug'};
+      $sDate = $1;
+      $state = $URL;
       }
 
+    elsif ($state eq $TITLE)
+      {
+      if (ref($hit))
+        {
+        my $sTitle = $_;
+        chomp $sTitle;
+        $sTitle =~ s/^\s+//;  # delete leading spaces
+        $hit->title($sTitle);
+        } # if
+      $state = $FORUM;
+      } # if TITLE
+
     elsif ($state eq $FORUM &&
-           m|Forum</b>:\s([^<]+)|i)
+           m|<td>([^<]+?)</td>|i)
       {
       print STDERR "forum line\n" if 2 <= $self->{'_debug'};
       # Actual line of input is:
-      # 	<b>Forum</b>: rec.birds<br>
-      $sDescription .= "Newsgroup: $1";
-      $state = $DATE;
+      #     <td>rec.arts.sf.starwars.</td>
+      my $sForum = $1;
+      $sForum =~ s/\s+$//;  # delete trailing whitespace
+      $sDescription .= "Newsgroup: $sForum";
+      $state = $AUTHOR;
       }
-    elsif ($state eq $DATE &&
-           m|Date</b>:\s([^\s]+).*?Author</b>: (.*)|i)
+    elsif ($state eq $AUTHOR && m|\">(.*?)</a>|i)
       {
       # Actual line of input is:
-      # 	<b>Date</b>: 1998/08/20 <b>Author</b>: James Hanst
-      print STDERR "date and author\n" if 2 <= $self->{'_debug'};
-      $hit->change_date($1);
-      $sDescription .= "; Author: $2";
-      $hit->description($sDescription);
+      #     <td><a href="profile.xp?author=315f0ddb1fd3416693f002e3662d9640ae3e4d3b2bceb4bf6eb0da6a80c070068f1069a1e792e4577e8fd9a49eb1d898171e5ed278071e92d5&ST=QS&ee=1">ceasar         </a></td>
+      print STDERR "author\n" if 2 <= $self->{'_debug'};
+      $sDescription .= "; Author: $1";
       $state = $HITS;
       } # line is end of description
 
@@ -311,8 +373,9 @@ sub native_retrieve_some
     # End, no other pages (missed some tag somewhere along the line?)
     $self->{_next_url} = undef;
     }
-  if (defined($hit)) 
+  if (ref($hit)) 
     {
+    $hit->description($sDescription);
     push(@{$self->{cache}}, $hit);
     }
   
@@ -334,3 +397,15 @@ http://x1.dejanews.com/dnquery.xp?QRY=Martin+Thurn&ST=PS&defaultOp=OR&DBS=1&show
 URL to get "text-only" of an article:
 
 http://x10.dejanews.com/getdoc.xp?AN=365996516&CONTEXT=899408575.427622419&hitnum=8&fmt=raw
+
+new basic search URL 1999-12-06:
+
+http://www.deja.com/dnquery.xp?DBS=2&ST=MS&test=SA&QRY=Martin+Thurn&svcclass=dncurrent
+
+new power search URL 1999-12-06:
+
+http://www.deja.com/[ST_rn=ps]/qs.xp?ST=PS&svcclass=dnyr&QRY=boba+fett&defaultOp=OR&DBS=1&OP=dnquery.xp&LNG=ALL&subjects=&groups=&authors=&fromdate=&todate=&showsort=score&maxhits=100
+
+simplest:
+
+http://www.deja.com/qs.xp?QRY=boba+fett&defaultOp=OR&OP=dnquery.xp&LNG=ALL&showsort=score&maxhits=100
